@@ -1,5 +1,5 @@
 use crate::value::{Value, ValueOption, ValueOptionOwned};
-use crate::{EResult, Error};
+use crate::Error;
 use crate::{ItemStatus, IEID, OID};
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use std::hash::{Hash, Hasher};
@@ -50,6 +50,7 @@ impl FromStr for NodeStatus {
     }
 }
 
+/// submitted to RPL/NODE/<name>
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct NodeStateEvent {
@@ -85,6 +86,7 @@ impl<'de> Deserialize<'de> for NodeStatus {
     }
 }
 
+/// Submitted by services via the bus for local items
 #[derive(Debug, Clone, Serialize, Eq, PartialEq)]
 #[serde(deny_unknown_fields)]
 pub struct RawStateEvent<'a> {
@@ -119,6 +121,7 @@ impl<'a> RawStateEvent<'a> {
     }
 }
 
+/// Submitted by services via the bus for local items
 #[derive(Debug, Clone, Serialize, Deserialize, Eq, PartialEq)]
 #[serde(deny_unknown_fields)]
 pub struct RawStateEventOwned {
@@ -153,21 +156,25 @@ impl RawStateEventOwned {
     }
 }
 
+/// Submitted by the core via the bus for procesed local events
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct LocalStateEvent {
     pub status: ItemStatus,
     pub value: Value,
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub act: Option<usize>,
     pub ieid: IEID,
     pub t: f64,
 }
 
+/// Submitted by the core via the bus for processed remote events
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct RemoteStateEvent {
     pub status: ItemStatus,
     pub value: Value,
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub act: Option<usize>,
     pub ieid: IEID,
     pub t: f64,
@@ -176,7 +183,11 @@ pub struct RemoteStateEvent {
 }
 
 impl RemoteStateEvent {
-    pub fn from_local_state_event(event: LocalStateEvent, system_name: &str) -> Self {
+    pub fn from_local_state_event(
+        event: LocalStateEvent,
+        system_name: &str,
+        connected: bool,
+    ) -> Self {
         Self {
             status: event.status,
             value: event.value,
@@ -184,11 +195,33 @@ impl RemoteStateEvent {
             ieid: event.ieid,
             t: event.t,
             node: system_name.to_owned(),
-            connected: true,
+            connected,
         }
     }
 }
 
+/// Stored by the core
+#[derive(Debug, Clone, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub struct DbState {
+    pub status: ItemStatus,
+    pub value: Value,
+    pub ieid: IEID,
+    pub t: f64,
+}
+
+/// Processed by the core and some additional services
+#[derive(Debug, Clone, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub struct ReplicationState {
+    pub status: ItemStatus,
+    pub value: Value,
+    pub act: Option<usize>,
+    pub ieid: IEID,
+    pub t: f64,
+}
+
+/// Submitted by replication services for remote items
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct ReplicationStateEvent {
@@ -197,12 +230,24 @@ pub struct ReplicationStateEvent {
     pub act: Option<usize>,
     pub ieid: IEID,
     pub t: f64,
-    pub node: Option<String>,
+    pub node: String,
 }
 
-impl TryFrom<ReplicationInventoryItem> for ReplicationStateEvent {
+impl From<ReplicationStateEvent> for ReplicationState {
+    fn from(d: ReplicationStateEvent) -> Self {
+        Self {
+            status: d.status,
+            value: d.value,
+            act: d.act,
+            ieid: d.ieid,
+            t: d.t,
+        }
+    }
+}
+
+impl TryFrom<ReplicationInventoryItem> for ReplicationState {
     type Error = Error;
-    fn try_from(item: ReplicationInventoryItem) -> EResult<Self> {
+    fn try_from(item: ReplicationInventoryItem) -> Result<Self, Self::Error> {
         let v: Option<Value> = item.value.into();
         Ok(Self {
             status: item.status.unwrap_or_default(),
@@ -214,7 +259,6 @@ impl TryFrom<ReplicationInventoryItem> for ReplicationStateEvent {
             t: item
                 .t
                 .ok_or_else(|| Error::invalid_data(format!("Set time missing ({})", item.oid)))?,
-            node: None,
         })
     }
 }
@@ -236,12 +280,26 @@ impl ReplicationStateEvent {
             act,
             ieid,
             t,
-            node: Some(node.to_owned()),
+            node: node.to_owned(),
         }
     }
 }
 
-// helper object, all fields should be public
+impl From<ReplicationStateEvent> for RemoteStateEvent {
+    fn from(d: ReplicationStateEvent) -> Self {
+        Self {
+            status: d.status,
+            value: d.value,
+            act: d.act,
+            ieid: d.ieid,
+            t: d.t,
+            node: d.node,
+            connected: true,
+        }
+    }
+}
+
+/// Submitted by replication services to RPL/INVENTORY/<name> (as a list of)
 #[derive(Debug, Serialize, Deserialize, Clone)]
 #[serde(deny_unknown_fields)]
 pub struct ReplicationInventoryItem {
@@ -271,3 +329,32 @@ impl PartialEq for ReplicationInventoryItem {
         self.oid == other.oid
     }
 }
+
+/// full state with info, returned by item.state RPC functions, used in HMI and other apps
+#[derive(Debug, Serialize, Clone)]
+#[serde(deny_unknown_fields)]
+pub struct FullItemStateAndInfo<'a> {
+    #[serde(flatten)]
+    pub si: ItemStateAndInfo<'a>,
+    // full
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub meta: Option<&'a Value>,
+    pub enabled: bool,
+}
+
+/// short state with info, returned by item.state RPC functions, used in HMI and other apps
+#[derive(Debug, Serialize, Clone)]
+#[serde(deny_unknown_fields)]
+pub struct ItemStateAndInfo<'a> {
+    pub oid: &'a OID,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub status: Option<ItemStatus>,
+    #[serde(default, skip_serializing_if = "ValueOptionOwned::is_none")]
+    pub value: ValueOptionOwned,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub act: Option<usize>,
+    pub ieid: Option<IEID>,
+    pub t: Option<f64>,
+    pub node: &'a str,
+    pub connected: bool,
+}
