@@ -62,6 +62,11 @@ impl Registry {
     }
 }
 
+#[inline]
+fn default_workers() -> u32 {
+    1
+}
+
 /// Initial properties for services
 #[derive(Debug, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
@@ -76,6 +81,8 @@ pub struct Initial {
     core: CoreInfo,
     bus: BusConfig,
     config: Option<Value>,
+    #[serde(default = "default_workers")]
+    workers: u32,
 }
 
 impl Initial {
@@ -89,6 +96,7 @@ impl Initial {
         core_info: CoreInfo,
         bus: BusConfig,
         config: Option<&Value>,
+        workers: u32,
     ) -> Self {
         Self {
             config_version: SERVICE_CONFIG_VERSION,
@@ -100,6 +108,7 @@ impl Initial {
             core: core_info,
             bus,
             config: config.map(Clone::clone),
+            workers,
         }
     }
     #[inline]
